/**
 *  @file       GrassCutterChordPlayer.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      This performer uses multiple karplus instruments to play
 *  chords.
 *  @extends    Performer
 **/
public class GrassCutterChordPlayer extends Performer {
    KarplusSampler grassCutters[4];
    for(0 => int i; i < grassCutters.size(); i++) {
        grassCutters[i].path("./sounds/Higher pitched hand held lawn mower, idle at end.aif");
        grassCutters[i].gain(0.6);
    }

    // The note that each object will play
    int _notes[];

    fun void play_chord(float onVelocity, dur onDuration) {
        for(0 => int i; i < _notes.size(); i++) {
            grassCutters[i].freq(Std.mtof(_notes[i]));
            spork ~ grassCutters[i].noteOn(onVelocity, onDuration);
        }
    }

    fun void play_major(float onVelocity, dur onDuration) {
        [69, 73, 76] @=> _notes;
        this.play_chord(onVelocity, onDuration);
    }

    fun void play_minor(float onVelocity, dur onDuration) {
        [69, 72, 76] @=> _notes;
        this.play_chord(onVelocity, onDuration);
    }

    fun void play_minor_variation(float onVelocity, dur onDuration) {
        [71, 76, 79] @=> _notes;
        this.play_chord(onVelocity, onDuration);        
    }

    fun void play_resolve(float onVelocity, dur onDuration) {
        [60, 67, 72, 76] @=> _notes;
        this.play_chord(onVelocity, onDuration);
    }

    fun void stop(float offVelocity, dur offDuration) {
        for(0 => int i; i < grassCutters.size(); i++) {
            spork ~ grassCutters[i].noteOff(offVelocity, offDuration);
        }
    }

}