/**
 *  @file       KarplusSampler.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      A subclass of `Sampler` that feeds the sample into the
 *  karplus strong algorithm.
 *  @extends    Sampler
 **/
public class KarplusSampler extends Sampler {


    // sample rate
    second / samp => float SRATE;

    _buf =< outputsAll;

    // feedforward
    _buf => Delay delay => JCRev r => Envelope e => outputsAll;
    // feedback
    delay => OneZero lowpass => delay;

    // set gain
    .25 => _buf.gain;
    // set reverb mix
    .1 => r.mix;


    // set lowpass zero position
    -1 => lowpass.zero;
    // set feedback gain
    .99 => lowpass.gain;

    fun float freq(float aFreq) {
        aFreq => _freq;

        // set delay
        (SRATE / _freq)::samp => delay.delay;

        return this.freq();
    }

    fun void playNote(float onVelocity) {
        // HACK: super.playNote(onVelocity)
                if(_path == "") {
            <<< "ERROR: Sampler path has not been set." >>>;
            me.exit();
        }

        _buf.gain(onVelocity);
        _buf.pos(0);
        // ENDHACK

        e.duration(1::ms);
        e.keyOn();

    }

    fun void noteOn(float onVelocity, dur onDuration) {
        e.duration(onDuration);
        e.keyOn();
        return this.noteOn(onVelocity);
    }

    fun void noteOff(float offVelocity, dur offDuration) {
        e.duration(offDuration);
        e.keyOff();
        offDuration => now;
        loop(false);
        _buf.pos(_buf.samples());
    }

}