/**
 *  @file       Main.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/


/**
 *  @class      
 *  @extends    Score
 **/
public class Main extends Score {

    /**
     *  @class      
     *  @extends    Score.Movement
     **/
    class PercussionExperiment extends Score.Movement {

        MarkovPercussionPlayer m;

        RepetitivePercussionPlayer repetitivePercussion;

        EffectedPurrBreathing purr;

        GrassCutterChordPlayer cutterPlayer;

        Sampler amyLaughs[2];
        amyLaughs[0].path("./sounds/Cat and Amy - Amy Laugh.aif");
        amyLaughs[1].path("./sounds/Cat and Amy - Amy Laugh 2.aif");

        KarplusSampler glitter;
        glitter.path("./sounds/Percussive Key Glitch Highpass.aif");

        KarplusSampler marleyWhine;
        marleyWhine.path("./sounds/Whiny Cat 4.aif");
        marleyWhine.gain(0.25);


        fun void initialize() {
            m.score(this.score());
            cutterPlayer.score(this.score());
            repetitivePercussion.score(this.score());
            purr.modulator_freq(1::second/this.score().noteDurations["1"]/2);
        }

        fun void play() {
            this.pre_play();

            this.score().noteDurations["1"] => dur wholeNote;
            this.score().noteDurations["1/4"] => dur quarterNote;
            this.score().noteDurations["1/8"] => dur eighthNote;


            spork ~ purr.noteOn(1.0);
            wholeNote*4 => now;

            repetitivePercussion.play();

            spork ~ repetitivePercussion.tick_tock() @=> Shred @ repetitiveShred;
            wholeNote*2 => now;

            spork ~ repetitivePercussion.once_per_measure() @=> Shred @ repetitiveShred2;
            m.play();
            spork ~ m.play_subtle() @=> Shred @ markovPercussion;

            wholeNote*2 => now;
            markovPercussion.exit();
            spork ~ m.play_subtle_faster() @=> markovPercussion;

            wholeNote*2 => now;

            repetitiveShred.exit();
            spork ~ repetitivePercussion.double_tick_tock() @=> repetitiveShred;

            wholeNote*2 => now;

            cutterPlayer.play_major(0.05, wholeNote);

            wholeNote + quarterNote*6 => now;

            marleyWhine.freq(Std.mtof(76+12));
            marleyWhine.playNote(1.0);

            quarterNote => now;

            cutterPlayer.stop(1.0, wholeNote);

            wholeNote => now;

            cutterPlayer.play_minor(0.05, wholeNote);

            wholeNote + quarterNote*5 => now;

            marleyWhine.freq(Std.mtof(72+12));
            marleyWhine.playNote(1.0);
            quarterNote => now;

            marleyWhine.freq(Std.mtof(76+12));
            marleyWhine.playNote(1.0);
            quarterNote => now;

            cutterPlayer.stop(1.0, wholeNote);

            wholeNote => now;

            cutterPlayer.play_major(0.05, wholeNote);

            wholeNote => now;

            cutterPlayer.play_minor_variation(0.05, wholeNote);

            markovPercussion.exit();
            spork ~ m.play_subtle() @=> markovPercussion;

            wholeNote => now;

            cutterPlayer.play_resolve(0.05, wholeNote);

            wholeNote*2 => now;

            cutterPlayer.stop(1.0, wholeNote*2);

            this.score().noteDurations["1/4"] => now;
            repetitiveShred.exit();

            wholeNote => now;

            purr.noteOff();

            markovPercussion.exit();
            spork ~ m.play_solo() @=> markovPercussion;

            wholeNote => now;

            amyLaughs[0].playNote(0.25);

            wholeNote => now;

            amyLaughs[1].playNote(0.25);

            wholeNote => now;

            amyLaughs[1].playNote(0.25);

            this.remaining_duration() => now;
        }


    }

    this.bpm(125);
    this.time_signature(7);

    PercussionExperiment p;
    this.add_movement(p);
    // 32 measures
    p.duration(this.noteDurations["1"] * 28);
    p.initialize();

}

Main m;
m.play();