/**
 *  @file       MarkovChain.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
*  @class  1st order markov lookup table
 **/
public class MarkovChain {
    float seed;

    /**
     *  The markov lookup table of probabilities for each event.
     *
     *  Example:
     *                      Next pitch:
     *                  A       B       C             
     *  Previous    A  1/3     2/3     0/3
     *  pitch:      B  0/3     1/3     2/3
     *              C  2/3     0/3     1/3
     **/
    float _probabilities[][];

    fun float[][] probabilities(float someProbabilities[][]) {
        someProbabilities @=> _probabilities;


        for(0 => int i; i < _probabilities.size(); i++) {
            0 => float sum;
            for(0 => int j; j < _probabilities[i].size(); j++) {
                _probabilities[i][j] +=> sum;
            }

            if(sum > 1.0) {
                <<< "WARNING: Row", i, "sums to", sum, "." >>>;
            }
        }

        return _probabilities;
    }

    // for(0 => int i; i < _probabilities.size(); i++) {
    //     for(0 => int j; j < _probabilities[i].size(); j++) {
    //         <<< this.score().noteDurations[i], "->", this.score().noteDurations[j], ":", _probabilities[i][j] >>>;
    //     }
    // }

    /**
     *  The index of the previous event.
     **/
    0 => int prevIndex;


    /**
     *  Provided that there is a previous event, search through that
     *  row for the largest probability that is less than the seed.
     **/
    fun int nextIndex() {
        // Generate random seed
        Std.fabs(Std.randf()) => seed;

        // Keep track of current largest probability value, and the column index
        -1.0 => float currentLargestProbability;
        int currentLargestProbabilityIndex;

        0 => int j;
        _probabilities[prevIndex] @=> float row[];
        
        // for each possible probability in row
        while(j < row.size()) {
            // if probability is a candidate
            if(row[j] < seed && row[j] >= currentLargestProbability) {
                // If it is equal to other candidate, randomly choose between current and new
                if(row[j] != currentLargestProbability || maybe) {
                    j => currentLargestProbabilityIndex;
                    row[j] => currentLargestProbability;
                }
            }
            j++;
        }

        currentLargestProbabilityIndex => prevIndex;
        return currentLargestProbabilityIndex;        
    }

}