/**
 *  @file       MarkovPercussionPlayer.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      Plays percussion samplers based on a 1st order markov table.
 *  @extends    Performer
 **/
public class MarkovPercussionPlayer extends Performer {
    Sampler samples[10];
    // Only the first two samples will be played
    2 => int _activeSamples;
    samples[0].path("./sounds/Accidental Grainy Percussive Reversed.aif");
    samples[0].gain(0.4);
    
    samples[1].path("./sounds/Accidental Grainy Percussive Reversed 2.aif");
    samples[1].gain(0.6);

    samples[2].path("./sounds/Percussive Key Glitch Highpass.aif");
    samples[2].gain(0.6);

    samples[3].path("./sounds/Accidental Grainy Percussive Reversed 3.aif");
    samples[4].path("./sounds/Percussive Key Jingle.aif");
    // samples[5].path("./sounds/Percussive Key Inserting.aif");

    MarkovChain markov;


    [
    /*      1       1/2     1/4     1/8     1/16    1/32    1/64
    /*1*/   [0/7.0, 0/7.0,  3/7.0,  2/7.0,  2/7.0,  0/7.0,  0/7.0],
    /*1/2*/ [0/7.0, 0/7.0,  2/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0],
    /*1/4*/ [0/7.0, 0/7.0,  2/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0],
    /*1/8*/ [0/7.0, 0/7.0,  1/7.0,  2/7.0,  3/7.0,  1/7.0,  0/7.0],
    /*1/16*/[0/7.0, 0/7.0,  1/7.0,  2/7.0,  3/7.0,  1/7.0,  0/7.0],
    /*1/32*/[0/7.0, 0/7.0,  1/7.0,  1/7.0,  3/7.0,  2/7.0,  0/7.0],
    /*1/64*/[0/7.0, 0/7.0,  1/7.0,  1/7.0,  2/7.0,  2/7.0,  1/7.0]
    ] @=> float soloRhythm[][];

    [
    /*      1       1/2     1/4     1/8     1/16    1/32    1/64
    /*1*/   [1/7.0, 2/7.0,  4/7.0,  0/7.0,  0/7.0,  0/7.0,  0/7.0],
    /*1/2*/ [2/7.0, 1/7.0,  4/7.0,  0/7.0,  0/7.0,  0/7.0,  0/7.0],
    /*1/4*/ [0/7.0, 2/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0,  0/7.0],
    /*1/8*/ [1/7.0, 1/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0,  0/7.0],
    /*1/16*/[0/7.0, 1/7.0,  1/7.0,  2/7.0,  3/7.0,  0/7.0,  0/7.0],
    /*1/32*/[0/7.0, 1/7.0,  1/7.0,  1/7.0,  2/7.0,  2/7.0,  0/7.0],
    /*1/64*/[1/7.0, 1/7.0,  1/7.0,  1/7.0,  1/7.0,  2/7.0,  0/7.0]
    ] @=> float subtleRhythm[][];

    [
    /*      1       1/2     1/4     1/8     1/16    1/32    1/64
    /*1*/   [0/7.0, 1/7.0,  4/7.0,  2/7.0,  0/7.0,  0/7.0,  0/7.0],
    /*1/2*/ [0/7.0, 0/7.0,  3/7.0,  3/7.0,  1/7.0,  0/7.0,  0/7.0],
    /*1/4*/ [0/7.0, 0/7.0,  2/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0],
    /*1/8*/ [0/7.0, 0/7.0,  3/7.0,  3/7.0,  1/7.0,  0/7.0,  0/7.0],
    /*1/16*/[0/7.0, 0/7.0,  2/7.0,  3/7.0,  2/7.0,  0/7.0,  0/7.0],
    /*1/32*/[0/7.0, 1/7.0,  1/7.0,  1/7.0,  2/7.0,  2/7.0,  0/7.0],
    /*1/64*/[1/7.0, 1/7.0,  1/7.0,  1/7.0,  1/7.0,  2/7.0,  0/7.0]
    ] @=> float subtleFasterRhythm[][];

    fun void _play_markov() {
        while(true) {

            samples[Std.rand2(0, _activeSamples-1)].playNote(1.0);

            this.score().noteDurations[markov.nextIndex()] => now;
        }        
    }


    fun void play_subtle() {
        markov.probabilities(subtleRhythm);
        this._play_markov();
    }

    fun void play_subtle_faster() {
        markov.probabilities(subtleFasterRhythm);
        5 => _activeSamples;
        this._play_markov();
    }

    fun void play_solo() {
        markov.probabilities(soloRhythm);
        5 => _activeSamples;
        this._play_markov();
    }



    fun void play() {
        this.pre_play();

        // while(true) {
        //     // Std.rand2(0, 1) => int samplerIndex;
        //     markov.nextIndex() => int durationIndex;
        //     Std.rand2f(0.2, 5.0) => float samplerRate;

        //     s[0].rate(samplerRate);
        //     spork ~ s[0].playNote(1.0);
        //     this.score().noteDurations[durationIndex] => now;
        // }


        // for(0 => int i; i < 4; i++) {
        //     s.playNote(1.0);
        //     this.score().quarterNote => now;
        // }

        // for(0 => int i; i < 8; i++) {
        //     s.playNote(1.0);
        //     this.score().eighthNote => now;
        // }



        // this.remaining_duration() => now;
    }
}