/**
 *  @file       RepetitivePercussionPlayer.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      Composed beats that can be repeated using 3 filtered sounds.
 *  @extends    Performer
 **/
public class RepetitivePercussionPlayer extends Performer {

    FilteredSampler s[3];
    s[0].path("./sounds/Percussive Key Inserting Reversed.aif");
    // s.path("./sounds/Percussive Key Jingle.aif");

    s[1].path("./sounds/Percussive Key Glitch Highpass.aif");

    s[2].path("./sounds/Percussive Key Inserting.aif");
    s[2].gain(0.8);

    Sampler keyGlitch;
    keyGlitch.path("./sounds/Percussive Key Glitch Highpass.aif");
    keyGlitch.gain(0.33);

    for(0 => int i; i < s.size(); i++) {
        s[i].gain(0.8);
    }

    fun void basic_44_beat() {
        s[0].rate(0.10);
        s[0].filterFreq(250);
        s[0].filterQ(0.5);
        s[1].filterFreq(1000);
        s[1].filterQ(10);

        while(true) {
            spork ~ s[0].playNote(1.0);
            this.score().noteDurations["1/4"] => now;
            spork ~ s[1].playNote(1.0);
            this.score().noteDurations["1/4"]+this.score().noteDurations["1/8"] => now;
            spork ~ s[1].playNote(1.0);
            this.score().noteDurations["1/8"] => now;
            spork ~ s[1].playNote(1.0);

            this.score().noteDurations["1/4"] => now;
        }        
    }

    /**
     *  Plays an alternating "tick/tock" clicky sound on every 1/8th note.
     **/
    fun void tick_tock() {
        4 => float rate;
        while(true) {
            if(rate == 4) {
                2 => rate;
            }
            else {
                4 => rate;
            }
            keyGlitch.rate(rate);
            spork ~ keyGlitch.playNote(1.0);
            this.score().noteDurations["1/8"] => now;
        }
    }

    fun void double_tick_tock_variation() {
        while(true) {
            if(maybe) {
                for(0 => int i; i < 14; i++) {
                    spork ~ keyGlitch.playNote(1.0);
                    this.score().noteDurations["1/32"] => now;
                }
            }

            this.score().noteDurations["1"] => now;            
        }
    }

    fun void double_tick_tock() {
        spork ~ double_tick_tock_variation();

        4 => float rate;
        while(true) {
            if(rate == 4) {
                2 => rate;
            }
            else {
                4 => rate;
            }
            keyGlitch.rate(rate);
            spork ~ keyGlitch.playNote(1.0);
            this.score().noteDurations["1/16"] => now;
            spork ~ keyGlitch.playNote(1.0);
            this.score().noteDurations["1/16"] => now;
        }
    }

    /**
     *  Plays grainy key inserting sound every 7 quarter notes.
     **/
    fun void once_per_measure() {
        s[2].rate(0.20);
        s[2].filterFreq(440*4);
        s[2].filterQ(7);
        s[2].gain(0.33);

        while(true) {
            s[2].playNote(1.0);      
            this.score().noteDurations["1/4"]*7 => now;
        }
        
    }

    fun void play() {
        this.pre_play();

        // once_per_measure();

        // while(true) {
        //     1::second => now;
        // }
    }
    

}