/**
 *  @file       Sampler.ck
 *
 *  @author     Colin Sullivan <colinsul [at] gmail.com>
 *
 *              Copyright (c) 2012 Colin Sullivan
 *              Licensed under the MIT license.
 **/

/**
 *  @class      Base sampler class
 *  @extends    Instrument
 **/
public class Sampler extends Instrument {
    SndBuf _buf => outputsAll;

    /**
     *  Wether or not we've told the SndBuf to loop.
     **/
    0 => int _looping;
    fun int loop() {
        return _looping;
    }
    fun int loop(int aValue) {
        aValue => _looping;
        _buf.loop(_looping);
        return loop();
    }

    "" => string _path;
    fun string path(string aPath) {
        _buf.read(aPath);
        _buf.pos(_buf.samples());

        aPath => _path;
        return path();
    }

    fun string path() {
        return _path;
    }

    fun float rate(float aRate) {
        return _buf.rate(aRate);
    }

    fun void playNote(float onVelocity) {
        if(_path == "") {
            <<< "ERROR: Sampler path has not been set." >>>;
            me.exit();
        }

        _buf.gain(onVelocity);
        _buf.pos(0);
    }

    fun void noteOn(float onVelocity) {
        this.loop(true);
        return this.playNote(onVelocity);
    }

    fun void noteOff() {
        this.loop(false);
        _buf.pos(_buf.samples());
    }

}